               page      60,132
NAME           bootsave
               title     BootSave save a copy of the Master boot sector
Comment |
        Version 2.7 Roedy Green
        works with MASM 6.0 and Optasm
See Boot.TXT for details on use.
  USAGE:
Examples:
*********
BootSave A:\MyBoot.Sav /Q
BootRest A:\MyBoot.Sav /Q
BootChk  A:\ByBoot.Sav /Q
These are DOS utilities, and thus can only use short 8.3 file names.
Syntax errors or missing file trouble generates an ERRORLEVEL 4.
BootChk generates an ERRORLEVEL 1 if the Boot record has changed
since the BootSave was done.
Version History:
 1.0 1990-11-19 released to BIX
 1.1 1991-09-23 released to BIX
                simpler use, parallels CMOSSave
 1.2 1991-09-24 ypassed to keep in sync with CMOSSAVE
 1.3 1991-09-25 change status to shareware
                banner changes
                error message beeps
 1.4 1993-06-01 new phone number and address
 1.5 1995-08-08 /Q option
                cleaner CHKBOOT.BAT
                configure with SET CMPDIR= to say where utils and *.SAV files kept.
 1.6 1996-10-23 Quadra address
                presume C:\SAFE again for files.
 1.7 1996-11-15 lower cost, combo site licence.
 1.8 1996-11-22 missing com files corrected
 1.9 1998-11-07 new mailing address and website.
 2.0 2001-01-08 simpler bat files.
 2.1 2002-01-15 simpler bat files.
 2.5 2008-01-02 add XML pad, icon
 2.6 2009-03-12 add ANT build script.
                refactor into three separate programs with common includes.
 2.7 2009-08-12 insert missing parse calls into bootchk and bootrest lost during reorg
 2.8 2009-08-20 refactor
 2.9 2012-11-25 minor chances to make avoid syntax errors from slightly different version of MASM.
*/
How to Assemble
***************
Manually set the GENERATING equate in embedded in the following code, then:
to assemble with MASM 6.0 use:
ML.EXE /AT /c /Fl /VM /Zf BOOT.Asm
LINK.EXE /TINY /MAP BOOT.Obj,BOOT.com,BOOT.map;
copy boot.com bootsave.com
to assemble with OPTASM use:
Optasm   BOOT.Asm,BOOT.Obj,BOOT.Lst/L/N/G/S
OLINK    BOOT.Obj,BOOT.COM,/MAP/TINY;
copy boot.com bootsave.com
|                                       ; end of comment
               .286
               .model    tiny           ; this is a COM file
stack          segment   stack          ; keep MS link happy by providing null stack
stack          ends
CODE           segment   PARA           ; start off in code.
;==============================================================
data           segment   byte           ; provide a separate DATA segment
                                        ; actually all come after the code
;==============================================================
;  V A R I A B L E S
BannerMsg      db        ' BootSave 2.9 ۲',13d,10d
               db        13d,10d
               db        'Saves contents of hard disk master boot sector to a file on disk or floppy.',13,10
               db        'Copyright: (c) 1991-2017 Roedy Green, Canadian Mind Products.',13,10
               db        '#101 - 2536 Wark Street, Victoria, BC Canada V8T 4G8',13,10
               db        'Telephone:(250) 361-9093   mailto:roedyg@mindprod.com   http://mindprod.com',13,10
               db        'Freeware to freely distribute and use for any purpose except military.',13,10
               db        13,10
               db        '$'
UsageMsg       db        ' Error ۲',7,13,10
               db        'Insert a formatted diskette.',13,10
               db        'then try:',13,10
               db        'BootSav A:\Boot.Sav',13,10
               db        'or if want to save on hard disk try:',13,10
               db        'BootSav C:\SAFE\Boot.Sav',13,10
               db        'BootChk is a DOS utility, and thus can only use short 8.3 file names',13,10
               db        'Read Boot.Txt to find how to use it properly.',13,10
               db        '$'
FileTroubleMsg db        ' Error ۲',7,13,10
               db        'Cannot create the 516-byte bootsave disk file.',13,10
               db        '$'
BootTroubleMsg db        ' Error ۲',7,13,10
               db        'Cannot read the boot sector on hard disk.',13,10
               db        '$'
WorkedMsg      db        'Hard disk master boot sector successfully saved.',13,10
               db        '$'
ParmIndex      dw        0
Filename       db        64 dup (0)
                                        ; asciiz filename
SlashQuiet     db        0
                                        ; -1 means quiet mode /QQ
CXSave         dw        0              ; disk geometry from INT 13/08
DXSave         dw        0              ; will be saved along with boot record
Buffer         dw        0              ; dynamic buffer will grow to 512
                                        ; it hangs out past the end of the program
Buffer2        equ       Buffer+512
                                        ; used to compare two versions of sector
data           ends
com            group     code,data      ; force data segment to go at the end
               assume    CS:com,DS:com,ES:com,SS:com
                                        ; seg regs cover everything
               org       100H           ; in Code segment
;==========================
_Start   proc      near
;       M A I N L I N E   R O U T I N E
               call      Parse          ; get filename from command line
               lea       dx,BannerMsg   ; display the banner after have /Q
               call      SayQ
               call      GetBoot        ; fetch Boot to buffer
               call      SaveGeom       ; save disk geometry as well
               call      WriteBootToFile ; write Boot contents to file
               lea       dx,WorkedMsg   ; crow about success
               call      Say
Done:
               mov       ax,4c00h
               int       21h            ;normal termination
_Start        endp
;===============================================================
SaveGeom       proc      near
;       Save disk geometry (heads, cyls)
;       We can later ensure we are restoring to a suitable disk
               mov       dx,0080h       ; get drv parms for disk c:
               mov       ax,0800h       ; ah 08 = get disk parameters
               int       13h            ; into CX/DX
               jc        BootTrouble
               mov       CXSave,CX
               mov       DXSave,DX
               ret
SaveGeom       endp
;===============================================================
; handle various kinds of trouble
FileTrouble    proc      near
               lea       dx,FileTroubleMsg ; display file trouble
               call      Say
               jmp       Abort
FileTrouble    endp
BootTrouble    proc      near
               lea       dx,BootTroubleMsg ; display problems reading boot sector
               call      Say
               jmp       Abort
BootTrouble    endp
SyntaxTrouble  proc      near
               lea       dx,BannerMsg
               call      Say            ; always display banner on syntax error error
               lea       dx,UsageMsg    ; display usage message
               call      Say
               jmp       Abort
SyntaxTrouble  endp
abort          proc      near
                                        ; error exit
               mov       ax, 4c04h      ; ERRORLEVEL = 4
               int       21h            ; DIE
abort          endp
;===============================================================
WriteBootToFile proc     near
;       Create a boot.sav file write Boot to it
               lea       dx,FileName    ; DS:DX point to filename
               xor       cx,cx          ; CX=0 is attribute
               mov       ah,03ch        ; DOS create function
               int       21h
               jc        FileTrouble
               mov       bx,ax          ; save handle
               mov       cx,512+4       ; write CX/DXSave + 1 sector buffer
               lea       dx,CXSave      ; buffer address
               mov       ah,40h         ; DOS write
               int       21h
               jc        FileTrouble
               cmp       ax,512+4
               jne       FileTrouble
               mov       ah,3eh         ; DOS close
               int       21h
               jc        FileTrouble
               ret
WriteBootToFile endp
;===============================================================
include        bootinc0.asm
;===============================================================
CODE           ends                     ; end of code segment
               end       _Start
